{--------------------------------------------------------------------------------
* Description : Label with link to file/URL/email
* Dates : December 1999
* Version : 1.1

* Author : Stefan Cruysberghs
* Email : stefancr@scip.be
* Website : http://www.scip.be
  Visit SCIP.be for other components, tools, articles, ...
--------------------------------------------------------------------------------
* $Archive: /Component Library/SC_Public/scLinkLabel.pas $
* $Author: Stefancr $
* $Date: 19/06/03 15:59 $
* $Modtime: 4/09/02 19:37 $
* $Revision: 1 $
--------------------------------------------------------------------------------
* This component is free of charge.
* The author doesn't give a warranty for error free running
  of this component and he doesn't give any support.
* Suggestions and bugs can be send by email.
--------------------------------------------------------------------------------
Installation
* Install the TscLinkLabel component by adding the unit scLinkLabel.pas
  to a package. Remove the DCR file in the package. Compile and Install the package.
* The TscLinkLabel component can be found in the tabsheet 'SC'
  of the component palette.
-------------------------------------------------------------------------------}

unit scLinkLabel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Shellapi;

type
  TscLinkLabel = class(TCustomLabel)
  private
    FCursor : TCursor;
    FFontStandard : TFont;
    FFontHighlight : TFont;
    FFontUsed : TFont;
    FBlnUsed : Boolean;
    FStrLink : String;
    FStrViewer : String;
    FStrParameters : String;
    FStrExtraInfo : String;
  protected
    procedure Click; override;
    procedure MouseOver(var msg : Tmessage); message CM_MOUSEENTER;
    procedure MouseLeave(var msg : Tmessage); message CM_MOUSELEAVE;
    procedure SetFontHighlight(Value: TFont);
    procedure SetFontUsed(Value: TFont);
    procedure SetFontStandard(const Value: TFont);
    procedure SetUsed(const Value: Boolean);
  public
    constructor Create(AOwner : Tcomponent); override;
    destructor Destroy; override;
  published
    // Is link already used ?
    property Used : Boolean read FBlnUsed write SetUsed default False;
    // Standard font
    property FontStandard : TFont read FFontStandard write SetFontStandard;
    // Font when label is highlighted
    property FontHighlight : TFont read FFontHighlight write SetFontHighlight;
    // Font when link is used
    property FontUsed : TFont read FFontUsed write SetFontUsed;
    // Link :
    //  - email-address (@)
    //  - URL (http://)
    //  - file
    property Link : string read FStrLink write FStrLink;
    // Path to external viewer for file (= link)
    property Viewer : string read FStrViewer write FStrViewer;
    // Extra parameters when using viewer
    property Parameters : string read FStrParameters write FStrParameters;
    property ExtraInfo : string read FStrExtraInfo write FStrExtraInfo;

    property Anchors;
    property Caption;
    property Color;
    property ShowHint;
    property Transparent;

    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation

{$R scLinkLabel.dcr}

procedure Register;
begin
  RegisterComponents('SC', [TscLinkLabel]);
end;

//------------------------------------------------------------------------------
constructor TscLinkLabel.Create(AOwner: Tcomponent);
begin
  inherited create(AOwner);
  FFontStandard := TFont.Create;
  FFontHighlight := TFont.Create;
  FFontHighlight.Color := clBlue;
  FFontHighlight.Style := [fsUnderline];
  FFontUsed := TFont.Create;
  FFontUsed.Color := clPurple;
  FBlnUsed:=False;
end;

//------------------------------------------------------------------------------
destructor TscLinkLabel.Destroy;
begin
  FreeAndNil(FFontStandard);
  FreeAndNil(FFontHighlight);
  FreeAndNil(FFontUsed);
  inherited;
end;

//------------------------------------------------------------------------------
procedure TscLinkLabel.Click;
begin
  if Trim(FStrLink) <> '' then
  begin
    // Email
    if Pos('@',FStrLink) <> 0 then
    begin
      ShellExecute(Application.Handle,nil,PChar('mailto:'+FStrLink),nil,nil,SW_SHOWNORMAL);
    end
    else
    begin
      // WWW
      if CompareText(Copy(FStrLink,1,4),'http') = 0 then
      begin
        ShellExecute(Application.Handle,nil,PChar(FStrLink),nil,nil,SW_SHOWNORMAL);
      end
      else
      // Files
      begin
        if FStrViewer <> '' then
          ShellExecute(Application.Handle,'open',
            PChar(ExtractFileName(FStrViewer)),
            PChar(FStrLink+' '+FStrParameters),
            PChar(ExtractFilePath(FStrViewer)),
            SW_SHOWNORMAL)
        else
          ShellExecute(Application.Handle,'open',
          PChar(ExtractFileName(FStrLink)),PChar(FStrParameters),
          PChar(ExtractFilePath(FStrLink)),
          SW_SHOWNORMAL)
      end;
    end;
  end;

  FBlnUsed:=True;
  Font.Assign(FFontUsed);
  inherited;
end;

//------------------------------------------------------------------------------
procedure TscLinkLabel.MouseLeave(var msg: Tmessage);
begin
  Cursor := FCursor;
  if FBlnUsed then
    Font.Assign(FFontUsed)
  else
    Font.Assign(FFontStandard);
end;

//------------------------------------------------------------------------------
procedure TscLinkLabel.MouseOver(var msg: Tmessage);
begin
  Font.Assign(FFontHighlight);
  FCursor := Cursor;
  if Trim(FStrLink) <> '' then
    Cursor := crHandPoint;
end;

//------------------------------------------------------------------------------
procedure TscLinkLabel.SetFontHighlight(Value: TFont);
begin
  FFontHighlight.Assign(Value);
end;

//------------------------------------------------------------------------------
procedure TscLinkLabel.SetFontUsed(Value: TFont);
begin
  FFontUsed.Assign(Value);
end;

//------------------------------------------------------------------------------
procedure TscLinkLabel.SetFontStandard(const Value: TFont);
begin
  FFontStandard.Assign(Value);
end;

//------------------------------------------------------------------------------
procedure TscLinkLabel.SetUsed(const Value: Boolean);
begin
  FBlnUsed := Value;
  if FBlnUsed then
    Font.Assign(FFontUsed)
  else
    Font.Assign(FFontStandard);  
end;

end.
